import { JitsuFunction } from "@jitsu/protocols/functions";
import type { AnalyticsServerEvent } from "@jitsu/protocols/analytics";
import { z } from "zod";

export const WebhookDestinationConfig = z.object({
  url: z.string(),
  method: z.enum(["GET", "POST", "PUT", "DELETE"]).default("POST"),
  headers: z.array(z.string()).optional(),
});

export type WebhookDestinationConfig = z.infer<typeof WebhookDestinationConfig>;

const WebhookDestination: JitsuFunction<AnalyticsServerEvent, WebhookDestinationConfig> = async (event, ctx) => {
  const headers = ctx.props.headers || [];
  await ctx.fetch(ctx.props.url, {
    method: ctx.props.method || "POST",
    body: JSON.stringify(event),
    headers: {
      "Content-Type": "application/json",
      ...headers.reduce((res, header) => {
        const [key, value] = header.split(":");
        return { ...res, [key]: value };
      }, {}),
    },
  });
  return event;
};

WebhookDestination.displayName = "webhook-destination";

export default WebhookDestination;
